@@ -16,6 +16,12 @@ alg_section = evp_properties
 rh-allow-sha1-signatures = yes
 '''
 
+FIPS_MODULE_CONFIG = '''
+[fips_sect]
+tls1-prf-ems-check = {}
+activate = 1
+'''
+
 
 class OpenSSLGenerator(ConfigGenerator):
     CONFIG_NAME = 'openssl'
@@ -209,7 +215,24 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         'RSA-PSS-RSAE-SHA2-384': 'rsa_pss_rsae_sha384',
         'RSA-PSS-RSAE-SHA2-512': 'rsa_pss_rsae_sha512',
         'EDDSA-ED25519': 'ed25519',
-        'EDDSA-ED448': 'ed448'
+        'EDDSA-ED448': 'ed448',
+    }
+
+    group_map = {
+        'SECP224R1': 'secp224r1',
+        'SECP256R1': 'secp256r1',
+        'SECP384R1': 'secp384r1',
+        'SECP521R1': 'secp521r1',
+        'X25519': 'X25519',
+        'X448': 'X448',
+        'FFDHE-2048': 'ffdhe2048',
+        'FFDHE-3072': 'ffdhe3072',
+        'FFDHE-4096': 'ffdhe4096',
+        'FFDHE-6144': 'ffdhe6144',
+        'FFDHE-8192': 'ffdhe8192',
+        'BRAINPOOL-P256R1': 'brainpoolP256r1',
+        'BRAINPOOL-P384R1': 'brainpoolP384r1',
+        'BRAINPOOL-P512R1': 'brainpoolP512r1',
     }
 
     @classmethod
@@ -232,10 +255,15 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
             s += 'DTLS.MaxProtocol ='
             s += f' {cls.protocol_map[policy.max_dtls_version]}\n'
 
-        sig_algs = [cls.sign_map[i]
-                    for i in p['sign'] if i in cls.sign_map]
+        sig_algs = [cls.sign_map[i] for i in p['sign'] if i in cls.sign_map]
         s += 'SignatureAlgorithms = ' + ':'.join(sig_algs) + '\n'
 
+        groups = [cls.group_map[i] for i in p['group'] if i in cls.group_map]
+        s += 'Groups = ' + ':'.join(groups) + '\n'
+
+        if policy.enums['__ems'] == 'RELAX':
+            s += 'Options = RHNoEnforceEMSinFIPS\n'
+
         if 'SHA1' in p['hash']:
             s += RH_ALLOW_SHA1
 
@@ -244,3 +272,21 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
     @classmethod
     def test_config(cls, config):  # pylint: disable=unused-argument
         return True
+
+
+class OpenSSLFIPSGenerator(ConfigGenerator):
+    CONFIG_NAME = 'openssl_fips'
+    SCOPES = {'tls', 'ssl', 'openssl'}
+
+    @classmethod
+    def generate_config(cls, policy):
+        # OpenSSL EMS relaxation is special
+        # in that it uses a separate FIPS module config
+        # and, just in case, EMS is enforcing by default.
+        # It only puts `= 0` there if it's explicitly relaxed.
+        # That's the reason why `__ems` is a tri-state enum.
+        return FIPS_MODULE_CONFIG.format(int(policy.enums['__ems'] != 'RELAX'))
+
+    @classmethod
+    def test_config(cls, config):  # pylint: disable=unused-argument
+        return True
